import React from 'react';
import * as stories from './TypeaheadInputMulti.stories';
import { composeStories, fireEvent, generateSnapshots, getTokens, render, screen, userEvent, } from '../../tests/helpers';
const { Default, Disabled } = composeStories(stories);
describe('<TypeaheadInputMulti>', () => {
    generateSnapshots(stories);
    it('renders a multi-select input with tokens', () => {
        const { container } = render(React.createElement(Default, null));
        expect(getTokens(container)).toHaveLength(3);
    });
    it('focuses the input', async () => {
        const user = userEvent.setup();
        const { container } = render(React.createElement(Default, null));
        if (container.firstElementChild) {
            await user.click(container.firstElementChild);
        }
        expect(screen.getByRole('textbox')).toHaveFocus();
    });
    it('does not focus a disabled input', async () => {
        const user = userEvent.setup();
        const { container } = render(React.createElement(Disabled, null));
        const input = screen.getByRole('textbox');
        expect(input).toBeDisabled();
        if (container.firstElementChild) {
            await user.click(container.firstElementChild);
        }
        expect(input).not.toHaveFocus();
        await user.click(input);
        expect(input).not.toHaveFocus();
    });
    it('prevents clicks on the input from bubbling', async () => {
        const onClick = jest.fn();
        render(React.createElement(Default, { onClick: onClick, value: "calif" }));
        const input = screen.getByRole('textbox');
        input.focus();
        input.selectionStart = 2;
        fireEvent.click(input);
        expect(onClick).toHaveBeenCalledTimes(1);
        expect(input.selectionStart).toBe(2);
    });
    it('calls the keydown handler', async () => {
        const user = userEvent.setup();
        const onKeyDown = jest.fn();
        render(React.createElement(Default, { onKeyDown: onKeyDown }));
        screen.getByRole('textbox').focus();
        await user.keyboard('{enter}');
        expect(onKeyDown).toHaveBeenCalledTimes(1);
    });
    it('focuses the last token', async () => {
        const user = userEvent.setup();
        const { container } = render(React.createElement(Default, null));
        screen.getByRole('textbox').focus();
        await user.keyboard('{backspace}');
        const tokens = getTokens(container);
        const lastToken = tokens[tokens.length - 1];
        expect(lastToken).toHaveFocus();
    });
    it('does not focus the last token when the input has a value', async () => {
        const user = userEvent.setup();
        render(React.createElement(Default, { value: "foo" }));
        const input = screen.getByRole('textbox');
        input.focus();
        await user.keyboard('{backspace}');
        expect(input).toHaveFocus();
    });
});
